﻿using SelfPropelledArtilleryUnit.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SelfPropelledArtilleryUnit.Drawnings;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение базового объекта-сущности
/// </summary>
public class DrawningPropelledArtillery
{
    /// <summary>
	/// Класс-сущность
	/// </summary>
	public EntityPropelledArtillery? EntityPropelledArtillery { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки cамоходной арт. установки
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки cамоходной арт. установки
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки cамоходной арт. установки
    /// </summary>
    private readonly int _drawningPropelledArtilleryWidth = 135;

    /// <summary>
    /// Высота прорисовки cамоходной арт. установки
    /// </summary>
    private readonly int _drawningPropelledArtilleryHeight = 105;

    /// <summary>
	/// Координата X объекта
	/// </summary>
	public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningPropelledArtilleryWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningPropelledArtilleryHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawningPropelledArtillery() 
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>

    public DrawningPropelledArtillery(int speed, double weight, Color bodyColor): this()
    {
        EntityPropelledArtillery = new EntityPropelledArtillery(speed, weight, bodyColor);
    }
    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningPropelledArtilleryWidth">Ширина прорисовки cамоходной арт. установки</param>
    /// <param name="drawningPropelledArtilleryHeight">Высота прорисовки cамоходной арт. установки</param>

    protected DrawningPropelledArtillery(int drawningPropelledArtilleryWidth, int drawningPropelledArtilleryHeight):this()
    {
        _drawningPropelledArtilleryWidth = drawningPropelledArtilleryWidth;
        _drawningPropelledArtilleryHeight = drawningPropelledArtilleryHeight;       
    }

    /// <summary>
	/// Установка границ поля
	/// </summary>
	/// <param name="width">Ширина поля</param>
	/// <param name="height">Высота поля</param>
	/// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
	public bool SetPictureSize(int width, int height)
    {
        if (_drawningPropelledArtilleryWidth > width || _drawningPropelledArtilleryHeight > height)
        {
            return false;
        }

        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX.HasValue || _startPosY.HasValue)
        {
            if (_startPosX + _drawningPropelledArtilleryWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawningPropelledArtilleryWidth;
            }
            else if (_startPosX < 0) _startPosX = 0;
            if (_startPosY + _drawningPropelledArtilleryHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawningPropelledArtilleryHeight;
            }
            else if (_startPosY < 0) _startPosY = 0;
        }
        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        if (x + _drawningPropelledArtilleryWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningPropelledArtilleryWidth;
        }
        else if (x < 0) _startPosX = 0;
        else _startPosX = x;

        if (y + _drawningPropelledArtilleryHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningPropelledArtilleryHeight;
        }
        else if (y < 0) _startPosY = 0;
        else _startPosY = y;
    }


    /// <summary>
	/// Изменение направления перемещения
	/// </summary>
	/// <param name="direction">Направление</param>
	/// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
	public bool MoveTransport(DirectionType direction)
    {
        if (EntityPropelledArtillery == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityPropelledArtillery.Step > 0)
                {
                    _startPosX -= (int)EntityPropelledArtillery.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityPropelledArtillery.Step > 0)
                {
                    _startPosY -= (int)EntityPropelledArtillery.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX + _drawningPropelledArtilleryWidth + EntityPropelledArtillery.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityPropelledArtillery.Step;
                }
                return true;
            // вниз
            case DirectionType.Down:
                if (_startPosY + _drawningPropelledArtilleryHeight + EntityPropelledArtillery.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityPropelledArtillery.Step;
                }
                return true;
            default:
                return false;
        }
    }


    /// <summary>
	/// Прорисовка объекта
	/// </summary>
	/// <param name="g"></param>
	public virtual void DrawTransport(Graphics g)
    {
        if (EntityPropelledArtillery == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }


        Pen pen = new(Color.Black);
        Brush brush = new SolidBrush(EntityPropelledArtillery.BodyColor);
        Brush brSlateGray = new SolidBrush(Color.SlateGray);

        g.DrawEllipse(pen, _startPosX.Value + 0, _startPosY.Value + 75, 30, 30);
        g.DrawEllipse(pen, _startPosX.Value + 90, _startPosY.Value + 75, 30, 30);
        g.DrawRectangle(pen, _startPosX.Value + 15, _startPosY.Value + 75, 90, 30);

        //границы ЦВЕТ
        g.FillRectangle(brush, _startPosX.Value + 25, _startPosY.Value + 40, 35, 30);//башня
        g.FillEllipse(brush, _startPosX.Value + 0, _startPosY.Value + 75, 30, 30);
        g.FillEllipse(brush, _startPosX.Value + 90, _startPosY.Value + 75, 30, 30);
        g.FillRectangle(brush, _startPosX.Value + 15, _startPosY.Value + 75, 90, 30);

        // границы арт. установки
        g.DrawRectangle(pen, _startPosX.Value + 25, _startPosY.Value + 40, 35, 30);
        g.DrawRectangle(pen, _startPosX.Value + 0, _startPosY.Value + 65, 120, 13);//крыша

        // верхние катки ЦВЕТ
        g.FillEllipse(brSlateGray, _startPosX.Value + 30, _startPosY.Value + 70, 20, 20);
        g.FillEllipse(brSlateGray, _startPosX.Value + 50, _startPosY.Value + 70, 20, 20);
        g.FillEllipse(brSlateGray, _startPosX.Value + 70, _startPosY.Value + 70, 20, 20);

        // верхние катки ОТРИСОВКА
        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 70, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 50, _startPosY.Value + 70, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 70, _startPosY.Value + 70, 20, 20);

        //нижние катки ЦВЕТ
        Brush brDimGray = new SolidBrush(Color.DimGray);
        g.FillEllipse(brDimGray, _startPosX.Value + 30, _startPosY.Value + 90, 15, 15);
        g.FillEllipse(brDimGray, _startPosX.Value + 45, _startPosY.Value + 90, 15, 15);
        g.FillEllipse(brDimGray, _startPosX.Value + 60, _startPosY.Value + 90, 15, 15);
        g.FillEllipse(brDimGray, _startPosX.Value + 75, _startPosY.Value + 90, 15, 15);

        //нижние катки ОТРИСОВКА
        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 90, 15, 15);
        g.DrawEllipse(pen, _startPosX.Value + 45, _startPosY.Value + 90, 15, 15);
        g.DrawEllipse(pen, _startPosX.Value + 60, _startPosY.Value + 90, 15, 15);
        g.DrawEllipse(pen, _startPosX.Value + 75, _startPosY.Value + 90, 15, 15);

        g.FillEllipse(brSlateGray, _startPosX.Value + 3, _startPosY.Value + 78, 24, 24);
        g.FillEllipse(brSlateGray, _startPosX.Value + 93, _startPosY.Value + 78, 24, 24);

        //Большие катки ОТРИСОВКА
        g.DrawEllipse(pen, _startPosX.Value + 3, _startPosY.Value + 78, 24, 24);
        g.DrawEllipse(pen, _startPosX.Value + 93, _startPosY.Value + 78, 24, 24);

        g.FillRectangle(brush, _startPosX.Value + 0, _startPosY.Value + 65, 120, 13);//

     

    }
}
